module Graph
  class GraphPerspective
    attr_reader :local_node
    def initialize(node)
      @local_node = node
      @priority_queue = []
    end
    
    def add_edge(nodeA, nodeB)
      if !nodeA.knows?(nodeB)
        # puts "Connecting #{nodeA} and #{nodeB}"
        nodeA.neighbors[nodeB.identity] = nodeB
        nodeB.neighbors[nodeA.identity] = nodeA
      end
    end
    
    def remove_edge(nodeA, nodeB)
      if nodeA.knows?(nodeB)
        # puts "Disconnecting #{nodeA} and #{nodeB}"
        nodeA.neighbors.delete(nodeB.identity)
        nodeB.neighbors.delete(nodeA.identity)
      end
    end
    
    def pick
      if @priority_queue.empty?
        dbg = @local_node.neighbors.values.sort
        # puts "Picked from neigbors: #{dbg.inspect}"
        dbg.first
      else
        dbg = @priority_queue.sort!
        # puts "Picked from queue: #{dbg.inspect}"
        dbg.pop
      end
    end
    
    def update_with(remoteNode, list)
      # puts "Calling Update with : #{remoteNode} #{list.inspect}"

      neighbors = list.inject({}) do |map, ident|
        map[ident] = Node[ident]; map
      end
      
      remoteNode.neighbors.each do |ident, node|
        if neighbors.key?(ident)
          if !@local_node.knows?(node)
            # puts "I didn't know about #{node}"
            @priority_queue << node
          end
          neighbors.delete(ident)
        else
          if @local_node.knows?(node)
            # puts "It's possible #{node} went down"
            node.jeopardy!
            @priority_queue << node
          end
          remove_edge(remoteNode, node)
        end
      end
      
      neighbors.each do |ident, node|
        if !@local_node.knows?(node)
          # puts "I aldo didn't know about #{node}"
          @priority_queue << node
        end
        add_edge(remoteNode, node)
      end
      
      add_edge(@local_node, remoteNode)
    end
    
    def dump
      Node.each do |node|
        node.dump
      end
      puts
    end
  end
end