module Graph
  class Node
    attr_accessor :jeopardy, :last_update, :neighbors
    attr_reader :host, :port, :identity
    
    @@nodes = {}
    def self.[](ident)
      @@nodes[ident] ||= Node.new(ident)
    end
    
    def self.each(&block)
      @@nodes.values.each(&block)
    end
    
    def initialize(ident)
      @identity = ident
      @neighbors = {}
      @jeopardy = false
      @last_update = Time.at(0)
      @host, @port = ident.split(":")
      @port = @port.to_i
      
      @@nodes[ident] = self
    end
    
    def <=>(other)
      if other.jeopardy? == self.jeopardy?
        self.last_update <=> other.last_update
      else
        self.jeopardy? ? 1 : -1
      end
    end
    
    def jeopardy?
      !!@jeopardy
    end
    
    def jeopardy!
      @jeopardy = true
    end
    
    def to_s
      @identity
    end
    
    def knows?(node)
      node.identity == @identity || @neighbors.key?(node.identity)
    end
    
    def inspect
      %{"#{to_s} (#{@last_update.to_i}) (#{@jeopardy.inspect})"}
    end
    
    def dump
      puts "#{to_s}: #{@neighbors.values.join(", ")}"
    end
  end
end