package graph.Gossip

import org.apache.thrift.TException
import org.apache.thrift.protocol.TBinaryProtocol
import org.apache.thrift.transport.{TSocket,TFramedTransport,TTransportException}

import graph.{GraphPerspective,Node}
import NodeHelper._

class GossipConnection(node:Node, graphPerspective:GraphPerspective) {
  val localNode = graphPerspective.localNode
  var erroredOut = false
  
  lazy val socket = new TSocket(node.host, node.port, 1000)
  lazy val transport = new TFramedTransport(socket)
  lazy val client = {
    val iProtocol = new TBinaryProtocol(transport)
    val oProtocol = new TBinaryProtocol(transport)
    
    transport.open
    new thrift.Graph.Client(iProtocol, oProtocol)
  }
  
  def sendList {
    try {
      if (!socket.isOpen && erroredOut) { socket.open }
      client.gossip(localNode, localNode.neighbors)
      node.updated
    } catch {
      case e => 
        erroredOut = true
        graphPerspective.removeEdge(localNode, node)
        socket.close
    }
  }
}