package graph.Gossip

import java.net.InetSocketAddress;
import java.net.ServerSocket;
import java.net.SocketException;

import scala.actors.Actor._

import org.apache.thrift.TBase
import org.apache.thrift.TException
import org.apache.thrift.TSerializer
import org.apache.thrift.TDeserializer
import org.apache.thrift.protocol.TBinaryProtocol
import org.apache.thrift.protocol.TProtocol
import org.apache.thrift.protocol.TProtocolFactory
import org.apache.thrift.protocol.TSimpleJSONProtocol
import org.apache.thrift.transport.TNonblockingServerSocket
import org.apache.thrift.transport.TTransport
import org.apache.thrift.server.{TServer,TNonblockingServer}

import graph.{GraphPerspective,Node}
import NodeHelper._

class Gossiper(graphPerspective:GraphPerspective) {
  val localNode = graphPerspective.localNode
  var running = true
  var connections = Map[Node,GossipConnection]()
  
  object Ticker extends Runnable {
    var delay = 1000
    var block: Unit => Unit = null
    def run {
      while (running) {
        block()
        Thread.sleep(delay)
      }
    }
    
    def apply(delay:Int, block:Unit => Unit) {
      this.delay = delay
      this.block = block
      new Thread(this).start
    }
  }
  
  object Listener extends Runnable {
    var server:TServer = null
    def run {
      this.server.serve
      
      stop
    }
    
    def apply(port:Int, handler:thrift.Graph.Iface) {
      val transport = new TNonblockingServerSocket(port)
      val processor = new thrift.Graph.Processor(handler)
      val protoFactory = new TBinaryProtocol.Factory(true, true)

      this.server = new TNonblockingServer(processor, transport)
      new Thread(this).start
    }
  }
  
  def run {
    Listener(localNode.port, new GossipHandler(actor {
      loopWhile(running) {
        reactWithin(5000) {
          case Gossip(node, list) => 
            graphPerspective.updateWith(node, list)
          case _TIMEOUT => 
        }
      }
    }))
    
    Ticker(1000, tick)    
  }
  
  def stop {
    running = false
  }
  
  def tick(u:Unit) = {
    val remoteNode = graphPerspective.pick
    if (remoteNode != null) {
      connectionFor(remoteNode).sendList
    }
  }
  
  def connectionFor(node:Node) = synchronized {
    if (!connections.contains(node)) {
      val connection = new GossipConnection(node, graphPerspective)
      connections = connections.updated(node, connection)
    }
    connections(node)
  }
}
