package graph

import scala.collection.mutable.PriorityQueue

import org.eclipse.jetty.util.ajax.JSON

class GraphPerspective(identity:String) extends JSON.Generator {

  var nodeQueue = new PriorityQueue[Node]()

  val localNode = Node(identity)
  
  def seed(node:Node) = nodeQueue.enqueue(node)
  
  def pick = {
    if (!nodeQueue.isEmpty)
      nodeQueue.dequeue
    else if (!localNode.neighbors.isEmpty)
      localNode.neighbors.values.max
    else null
  }

  def addEdge(nodeA:Node, nodeB:Node) {
    if(!nodeA.sees(nodeB)) {
      GraphServer.logger info "Connecting node " + nodeA + " and " + nodeB
    
      nodeA.connect(nodeB)
      nodeB.connect(nodeA)
    }
  }

  def removeEdge(nodeA:Node, nodeB:Node) {
    if (nodeA.sees(nodeB)) {
      GraphServer.logger info "Disconnecting " + nodeA + " and " + nodeB

      nodeA.disconnect(nodeB)
      nodeB.disconnect(nodeA)
    }
  }

  def updateWith(remoteNode:Node, list:List[Node]) {
    var neighbors = list.foldLeft(Map[String,Node]()) { (map, node) => map.updated(node.identity, node) }

    remoteNode.neighbors.foreach { (entry) =>
      val (ident, node) = entry
      if (neighbors.contains(ident)) {
        if (!localNode.sees(node) && !nodeQueue.contains(node)) {
          nodeQueue.enqueue(node)
        }
        neighbors -= ident
      } else {
        if (localNode.sees(node)) {
          node.inJeopardy = true
        }
        removeEdge(remoteNode, node)
      }
    }

    neighbors.foreach { (entry) =>
      val (ident, node) = entry
      if (!localNode.sees(node) && !nodeQueue.contains(node)) {
          nodeQueue.enqueue(node)
      }
      addEdge(remoteNode, node)
    }
    
    if (!localNode.sees(remoteNode)) {
      addEdge(localNode, remoteNode)
    }
  }
  
  def addJSON(buffer:Appendable) {
    var results = new java.util.HashMap[String,Node]()
    var queue = List(localNode)
    
    while (!queue.isEmpty) {
      val node = queue.head
      queue = queue.tail
      node.neighbors.values.foreach { (neighbor) => 
        if (!results.containsKey(neighbor.identity)) {
          results.put(neighbor.identity, neighbor)
          queue = neighbor :: queue
        }
      }
    }
    
    buffer.append(new JSON().toJSON(results))
  }
}