package graph

import java.util.logging.{Logger,FileHandler,SimpleFormatter,Level}
import org.eclipse.jetty.server.Server
import org.eclipse.jetty.server.handler.HandlerList
import org.eclipse.jetty.server.handler.DefaultHandler

import Gossip.Gossiper


object GraphServer {
  val GRAPH_SERVER_LOG_NAME = "GraphServerLog"

  lazy val logger = {
    val instance = Logger.getLogger(GRAPH_SERVER_LOG_NAME)
    instance.setLevel(Level.ALL)
    // TODO: configurable
    val handler = new FileHandler("/tmp/graph_server.log")
    handler.setFormatter(new SimpleFormatter)
    // TODO: configurable
    handler.setLevel(Level.ALL)
    instance.addHandler(handler)

    instance
  }

  def main(args: Array[String]) {
    val identity = args(0)

    logger.info("Initializing GraphPerspective for identity: " + identity)
    val graphPerspective = new GraphPerspective(identity)
    
    if (args.length >= 2) {
      logger.info("Seeding with " + args(1))
      graphPerspective.seed(Node(args(1)))
    }
    
    logger.info("Initializing Gossiper")
    
    val gossiper = new Gossiper(graphPerspective)
    gossiper.run
    if (args.length < 2) {
      val httpServer = new Server(8080)
      val handlerList = new HandlerList()
      handlerList.addHandler(new GraphWebSocket(graphPerspective))
      handlerList.addHandler(new GraphWebInterface())
      handlerList.addHandler(new DefaultHandler())
      httpServer.setHandler(handlerList)
 
      httpServer.start()
      httpServer.join()
    }
  }
}