package graph

import org.eclipse.jetty.util.ajax.JSON

class Node(val identity:String) extends JSON.Generator with Ordered[Node] {
  GraphServer.logger info "Creating node " + identity

  Node.add(this)

  var neighbors = Map[String,Node]()
  var lastUpdated = 0L
  var inJeopardy = false
  
  lazy val host = identity.split(":")(0)
  lazy val port = identity.split(":")(1).toInt

  override def compare(other:Node) =
    if (this.inJeopardy == other.inJeopardy) {
      other.lastUpdated.compare(this.lastUpdated)
    } else if (this.inJeopardy) {
      1
    } else {
      -1
    }
    
  override def toString = this.identity

  def updated {
    this.lastUpdated = System.currentTimeMillis
    this.inJeopardy = false
  }
  
  def addJSON(buffer:Appendable) {
    buffer.append(new JSON().toJSON(neighbors.values.toArray.map {_.toString}))
  }
  
  def connect(node:Node) = neighbors = neighbors.updated(node.identity, node)
  def disconnect(node:Node) = neighbors -= node.identity
  def sees(node:Node) = node == this || neighbors.contains(node.identity)
  def equals(other:Node) = this.identity.equals(other.identity)
}

object Node {
  private var nodes = Map[String,Node]()
  def apply(identity:String) = nodes.get(identity).getOrElse(new Node(identity))
  def add(node:Node) { nodes = nodes.updated(node.identity, node) }
}
